package reflect;

import java.lang.reflect.Method;

public class ReflectTest {

	public static void main(String[] args) throws Exception {

		// 根据字符串创建对象
		String clazz = "reflect.Circle";
		Class shapeClass = Class.forName(clazz);
		Circle cir = (Circle) makeObject(clazz);
		System.out.println(cir);

		// 根据字符串调用对象的方法(有参，无返回值)
		String methodName = "setRadius";// 该方法需要double型的参数setRadius(double radius);
		Class[] classTypes = { double.class };
		Object[] params = { 10 };
		invokeMethod(cir, methodName, classTypes, params);
		System.out.println(cir.getRadius());

		// 根据字符串调用对象的方法(无参，有返回值)
		String methodName1 = "getRadius";
		System.out.println("The result is " + invokeMethod(cir, methodName1, null, null));
	}

	/**
	 * @author zhrb
	 * @param className
	 *            代表类型名的字符串
	 * @return className类型的对象
	 * @throws Exception
	 *             NoClassDefFoundException或者没有默认构造器的异常
	 *             使用className代表的类的默认无参构造器，创建对象
	 */
	public static Object makeObject(String className) throws Exception {
		Class clazz = null;
		clazz = Class.forName(className);
		return clazz.newInstance();
	}

	/**
	 * @param o
	 *            调用方法的对象
	 * @param methodName
	 *            所要调用的方法名
	 * @param clazzes
	 *            方法所需要的参数的类型
	 * @param args
	 *            参数值
	 * @return 方法返回结果值
	 * @throws Exception
	 *             NoSuchMethoException,参数类型异常等
	 */
	public static Object invokeMethod(Object o, String methodName, Class[] clazzes, Object[] args) throws Exception {
		Method method = o.getClass().getMethod(methodName, clazzes);
		return method.invoke(o, args);
	}

}
